import React from 'react';
import purebem from 'purebem';
import { mount } from 'enzyme';

import Datepicker from './Datepicker';


const TestDatepicker = Datepicker({
    block: purebem.of('test-datepicker'),
    closeOnClickOutside: false,
    displayName: 'TestDatepicker'
});

const TestOnClickOutsideDatepicker = Datepicker({
    block: purebem.of('test-datepicker'),
    displayName: 'TestDatepicker'
});

test('should render empty text if no options', () => {
    const datepicker = mount(
        <TestDatepicker
            emptyText="Empty" />
    );

    expect(datepicker.text()).toBe('Empty');
});

[
    { name: 'should render date if passed as Date', data: { value: new Date(2000, 0, 1) } },
    { name: 'should render date if passed as timestamp', data: { value: 946681200000 } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const datepicker = mount(
            <TestDatepicker
                value={ fixture.data.value } />
        );

        expect(datepicker.text()).toBe('2000-01-01');
    });
});

test('should expand collapsed datepicker when clicked', () => {
    const datepicker = mount(
        <TestDatepicker
            value={ +new Date(2000, 0, 1) } />
    );

    datepicker.find('TestDatepickerControl').simulate('click');

    expect(datepicker.state('expanded')).toBe(true);
});

[
    { name: 'should expand collapsed datepicker when using enter', data: { which: 13 } },
    { name: 'should expand collapsed datepicker when using space', data: { which: 32 } },
    { name: 'should expand collapsed datepicker when using down arrow', data: { which: 40 } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const datepicker = mount(
            <TestDatepicker
                value={ +new Date(2000, 0, 1) } />
        );

        datepicker.simulate('keydown', { which: fixture.data.which });

        expect(datepicker.state('expanded')).toBe(true);
    });
});

test('should not expand collapsed datepicked when click and disabled', () => {
    const datepicker = mount(
        <TestDatepicker
            value={ +new Date(2000, 0, 1) }
            disabled={ true } />
    );

    datepicker.find('TestDatepickerControl').simulate('click');

    expect(datepicker.state('expanded')).toBe(false);
});

test('should collapse expanded dropdown when clicked', () => {
    const datepicker = mount(
        <TestDatepicker
            value={ +new Date(2000, 0, 1) } />
    );

    datepicker.find('TestDatepickerControl').simulate('click');
    datepicker.find('TestDatepickerControl').simulate('click');

    expect(datepicker.state('expanded')).toBe(false);
});

test('should collapse expanded dropdown when clicking outside', () => {
    const datepicker = mount(
        <TestOnClickOutsideDatepicker
            value={ +new Date(2000, 0, 1) } />
    );

    datepicker.simulate('click');
    document.dispatchEvent(new Event('mousedown'));

    expect(datepicker.find('TestOnClickOutsideDatepickerCalendar').length).toBe(0);
});

test('should collapse expanded dropdown when using escape', () => {
    const datepicker = mount(
        <TestDatepicker
            value={ +new Date(2000, 0, 1) } />
    );

    datepicker.find('TestDatepickerControl').simulate('click');
    datepicker.simulate('keydown', { which: 27 });

    expect(datepicker.state('expanded')).toBe(false);
});

[
    { name: 'should go to previous date when using left arrow when expanded', data: { which: 37 }, expected: { highlightedIndex: +new Date(2000, 0, 18) } },
    { name: 'should go to previous week when using up arrow when expanded', data: { which: 38 }, expected: { highlightedIndex: +new Date(2000, 0, 12) } },
    { name: 'should go to next date when using right arrow when expanded', data: { which: 39 }, expected: { highlightedIndex: +new Date(2000, 0, 20) } },
    { name: 'should go to next week when using down arrow when expanded', data: { which: 40 }, expected: { highlightedIndex: +new Date(2000, 0, 26) } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const datepicker = mount(
            <TestDatepicker
                value={ +new Date(2000, 0, 19) } />
        );

        datepicker.find('TestDatepickerControl').simulate('click');
        datepicker.simulate('keydown', { which: fixture.data.which });

        expect(datepicker.state('currentMonth')).toBe(+new Date(2000, 0, 1));
        expect(datepicker.state('highlightedIndex')).toBe(fixture.expected.highlightedIndex);
    });
});

[
    { name: 'should go to previous date/month when using left arrow when expanded', data: { which: 37 }, expected: { currentMonth: +new Date(2000, 0, 1), highlightedIndex: +new Date(2000, 0, 29) } },
    { name: 'should go to previous week/month when using up arrow when expanded', data: { which: 38 }, expected: { currentMonth: +new Date(2000, 0, 1), highlightedIndex: +new Date(2000, 0, 23) } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const datepicker = mount(
            <TestDatepicker
                value={ +new Date(2000, 1, 1) } />
        );

        datepicker.setState({
            highlightedIndex: +new Date(2000, 0, 30)
        });

        datepicker.find('TestDatepickerControl').simulate('click');
        datepicker.simulate('keydown', { which: fixture.data.which });

        expect(datepicker.state('currentMonth')).toBe(fixture.expected.currentMonth);
        expect(datepicker.state('highlightedIndex')).toBe(fixture.expected.highlightedIndex);
    });
});

[
    { name: 'should go to next date/month when using right arrow when expanded', data: { which: 39 }, expected: { currentMonth: +new Date(2000, 1, 1), highlightedIndex: +new Date(2000, 1, 6) } },
    { name: 'should go to next week/month when using down arrow when expanded', data: { which: 40 }, expected: { currentMonth: +new Date(2000, 1, 1), highlightedIndex: +new Date(2000, 1, 12) } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const datepicker = mount(
            <TestDatepicker
                value={ +new Date(2000, 0, 31) } />
        );

        datepicker.setState({
            highlightedIndex: +new Date(2000, 1, 5)
        });

        datepicker.find('TestDatepickerControl').simulate('click');
        datepicker.simulate('keydown', { which: fixture.data.which });

        expect(datepicker.state('currentMonth')).toBe(fixture.expected.currentMonth);
        expect(datepicker.state('highlightedIndex')).toBe(fixture.expected.highlightedIndex);
    });
});

test('should go to next month when using tab key when expanided', () => {
    const datepicker = mount(
        <TestDatepicker
            value={ +new Date(2000, 0, 19) } />
    );

    datepicker.find('TestDatepickerControl').simulate('click');
    datepicker.simulate('keydown', { which: 9 });

    expect(datepicker.state('currentMonth')).toBe(+new Date(2000, 1, 1));
    expect(datepicker.state('highlightedIndex')).toBe(+new Date(2000, 1, 19));
});

test('should go to previous month when using shift + tab key when expanded', () => {
    const datepicker = mount(
        <TestDatepicker
            value={ +new Date(2000, 0, 19) } />
    );

    datepicker.find('TestDatepickerControl').simulate('click');
    datepicker.simulate('keydown', { shiftKey: true, which: 9 });

    expect(datepicker.state('currentMonth')).toBe(+new Date(1999, 11, 1));
    expect(datepicker.state('highlightedIndex')).toBe(+new Date(1999, 11, 19));
});
