import React, { PropTypes } from 'react';
import ReactDOM from 'react-dom';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import onClickOutside from 'react-onclickoutside';
import invariant from 'invariant';
import moment from 'moment';

import * as lang from '../../utils/lang';
import FoundationTether from '../misc/FoundationTether';
import isNullOrEmpty from '../../utils/isNullOrEmpty';
import isHandheld from '../../utils/isHandheld';
import toTimestamp from '../../utils/toTimestamp';

import DatepickerCalendar from './DatepickerCalendar';
import DatepickerControl from './DatepickerControl';
import isVisibleInCurrentMonth from './utils/isVisibleInCurrentMonth';


export default ({
    block,
    closeOnClickOutside = true,
    displayName,
    uiPropDefaultType = PropTypes.string,
    uiPropDefaultValue = '',
    uiPropGetValue = (ui) => ui
}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');

    // These are extracted for better testability because I prefer to select based on
    // display name instead of CSS class.
    const Calendar = DatepickerCalendar({ block, displayName: `${displayName}Calendar` });
    const Control = DatepickerControl({ block, displayName: `${displayName}Control` });

    const Datepicker = React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            disabled: PropTypes.bool,
            emptyText: PropTypes.string,
            focus: PropTypes.bool,
            hasGrayedOutPast: PropTypes.bool,
            onChange: PropTypes.func,
            tetherTo: PropTypes.string,
            ui: uiPropDefaultType,
            value: PropTypes.oneOfType([
                PropTypes.instanceOf(Date),
                PropTypes.number
            ])
        },

        getDefaultProps() {
            return {
                disabled: false,
                emptyText: lang.pgettext('foundation-datepicker', 'Label--no-date-yet'),
                expanded: false,
                focus: false,
                hasGrayedOutPast: false,
                onChange: (/* value */) => {},
                tetherTo: 'content',
                ui: uiPropDefaultValue,
                value: null
            };
        },

        getInitialState() {
            const value = isNullOrEmpty(this.props.value) ? null : this.props.value;
            const highlightedIndex = +new Date(value === null ? +new Date() : value);

            const date = new Date(highlightedIndex);
            const currentMonth = +new Date(date.getFullYear(), date.getMonth(), 1);

            return {
                currentMonth,
                expanded: false,
                highlightedIndex
            };
        },

        componentDidMount() {
            if (this.props.focus) {
                ReactDOM.findDOMNode(this.component).focus();
            }
        },

        handleClickOutside() {
            if (this.state.expanded) {
                this.collapse();
            }
        },

        onChange(value) {
            this.change(value);
        },

        onCollapse() {
            this.collapse();
        },

        onKeyDown(evt) {
            let date, newCurrentMonth;
            let currentMonth = moment(this.state.currentMonth);

            switch (evt.which) {
                case 9: // tab
                    if (this.state.expanded) {
                        // don't tab to next form control when expanded.
                        evt.stopPropagation();
                        evt.preventDefault();

                        if (evt.shiftKey) {
                            this.moveToPreviousMonth();
                        } else {
                            this.moveToNextMonth();
                        }
                    }

                    break;
                case 13: // enter
                case 32: // space
                    evt.stopPropagation();
                    evt.preventDefault();

                    if (this.state.expanded) {
                        this.change(moment(this.state.highlightedIndex));
                    } else {
                        this.expand();
                    }

                    break;
                case 27: // escape
                    if (this.state.expanded) {
                        this.collapse();
                    }

                    break;
                case 37: // left
                    evt.preventDefault();

                    if (this.state.expanded) {
                        date = moment(this.state.highlightedIndex).subtract(1, 'days');
                        newCurrentMonth = isVisibleInCurrentMonth(currentMonth, date)
                            ? currentMonth
                            : currentMonth.clone().subtract(1, 'months');

                        this.changeHighlightedIndex(toTimestamp(newCurrentMonth), toTimestamp(date));
                    }

                    break;
                case 38: // up
                    evt.preventDefault();

                    if (this.state.expanded) {
                        date = moment(this.state.highlightedIndex).subtract(7, 'days');
                        newCurrentMonth = isVisibleInCurrentMonth(currentMonth, date)
                            ? currentMonth
                            : currentMonth.clone().subtract(1, 'months');

                        this.changeHighlightedIndex(toTimestamp(newCurrentMonth), toTimestamp(date));
                    }

                    break;
                case 39: // right
                    evt.preventDefault();

                    if (this.state.expanded) {
                        date = moment(this.state.highlightedIndex).add(1, 'days');
                        newCurrentMonth = isVisibleInCurrentMonth(currentMonth, date)
                            ? currentMonth
                            : currentMonth.clone().add(1, 'months');

                        this.changeHighlightedIndex(toTimestamp(newCurrentMonth), toTimestamp(date));
                    }

                    break;
                case 40: // down
                    evt.preventDefault();

                    if (this.state.expanded) {
                        date = moment(this.state.highlightedIndex).add(7, 'days');
                        newCurrentMonth = isVisibleInCurrentMonth(currentMonth, date)
                            ? currentMonth
                            : currentMonth.clone().add(1, 'months');

                        this.changeHighlightedIndex(toTimestamp(newCurrentMonth), toTimestamp(date));
                    } else {
                        this.expand();
                    }

                    break;
            }
        },

        onNextMonthClick() {
            this.moveToNextMonth();
        },

        onExpand() {
            this.expand();
        },

        onPreviousMonthClick() {
            this.moveToPreviousMonth();
        },

        onToggle() {
            if (this.props.disabled) {
                return;
            }

            this.state.expanded ? this.collapse() : this.expand();
        },

        onTouchableChange(evt) {
            if (evt.target.value === '') {
                this.change(null);
            } else {
                const date = moment(evt.target.value, 'YYYY-MM-DD');
                this.change(date);
            }
        },

        change(date) {
            const { disabled, onChange } = this.props;
            if (disabled) {
                return;
            }

            if (date === null) {
                const now = moment();
                this.setState({
                    currentMonth: toTimestamp(now.clone().startOf('month')),
                    expanded: false,
                    highlightedIndex: toTimestamp(now.clone())
                }, () => onChange(null));
            } else if (date.isValid()) {
                this.setState({
                    currentMonth: toTimestamp(date.clone().startOf('month')),
                    expanded: false,
                    highlightedIndex: toTimestamp(date.clone())
                }, () => onChange(toTimestamp(date.clone())));
            }
        },

        changeHighlightedIndex(currentMonth, highlightedIndex) {
            this.setState({ currentMonth, highlightedIndex });
        },

        collapse() {
            if (isHandheld()) {
                return;
            }

            const currentMonth = this.props.value !== null
                ? toTimestamp(moment(this.props.value).startOf('month'))
                : +new Date();

            const highlightedIndex = this.props.value !== null
                ? toTimestamp(moment(this.props.value))
                : +new Date();

            this.setState({
                currentMonth,
                expanded: false,
                highlightedIndex
            });
        },

        expand() {
            if (isHandheld()) {
                return;
            }

            this.setState({ expanded: true });
        },

        moveToNextMonth() {
            const { currentMonth, highlightedIndex } = this.state;

            this.changeHighlightedIndex(
                toTimestamp(moment(currentMonth).add(1, 'months')),
                toTimestamp(moment(highlightedIndex).add(1, 'months'))
            );
        },

        moveToPreviousMonth() {
            const { currentMonth, highlightedIndex } = this.state;

            this.changeHighlightedIndex(
                toTimestamp(moment(currentMonth).subtract(1, 'months')),
                toTimestamp(moment(highlightedIndex).subtract(1, 'months'))
            );
        },

        renderCalendar() {
            const { hasGrayedOutPast, tetherTo, value } = this.props;
            const { currentMonth, expanded, highlightedIndex } = this.state;

            if (!expanded || isHandheld()) {
                return null;
            }

            return (
                <FoundationTether
                    target={ () => ReactDOM.findDOMNode(this) }
                    tetherTo={ tetherTo }>
                    <Calendar
                        currentMonth={ moment(currentMonth) }
                        hasGrayedOutPast={ hasGrayedOutPast }
                        onNextMonthClick={ this.onNextMonthClick }
                        onPreviousMonthClick={ this.onPreviousMonthClick }
                        onChange={ this.onChange }
                        highlightedIndex={ moment(highlightedIndex) }
                        value={ moment(value) } />
                </FoundationTether>
            );
        },

        render() {
            const { disabled, ui } = this.props;
            const { expanded } = this.state;

            const touchable = isHandheld();

            const value = this.props.value !== null
                ? moment(this.props.value).format('YYYY-MM-DD')
                : this.props.emptyText;

            return (
                <div
                    className={ block({ disabled, expanded, touchable }, uiPropGetValue(ui)) }
                    onKeyDown={ this.onKeyDown }
                    onClick={ this.onToggle }
                    ref={ (c) => this.component = c }
                    tabIndex={ disabled ? null : '0' }>
                    <Control
                        defaultValue={ this.props.value !== null ? value : null }
                        disabled={ disabled }
                        onTouchableBlur={ this.onCollapse }
                        onTouchableChange={ this.onTouchableChange }
                        onTouchableFocus={ this.onExpand }
                        value={ value } />
                    { this.renderCalendar() }
                </div>
            );
        }

    });

    return closeOnClickOutside
        ? onClickOutside(Datepicker)
        : Datepicker;

};
