import React from 'react';
import purebem from 'purebem';
import { mount } from 'enzyme';

import Dropdown from './Dropdown';

import dropdownValueOptionState from './option-value/dropdownValueOptionState';


const TestDropdown = Dropdown({
    block: purebem.of('test-dropdown'),
    closeOnClickOutside: false,
    displayName: 'TestDropdown'
});

const TestOnClickOutsideDropdown = Dropdown({
    block: purebem.of('test-dropdown'),
    displayName: 'TestDropdown'
});


const getOptions = (a = false, b = false, c = false) => {
    return [
        dropdownValueOptionState({
            disabled: a,
            text: 'A',
            value: 'a'
        }),
        dropdownValueOptionState({
            disabled: b,
            text: 'B',
            value: 'b'
        }),
        dropdownValueOptionState({
            disabled: c,
            text: 'C',
            textSelected: 'C (selected)',
            value: 'c'
        })
    ];
};

const expandAt = (dropdown, highlightedIndex = -1) => {
    dropdown.setState({
        expanded: true,
        highlightedIndex
    });
};

test('should render empty text if no options', () => {
    const dropdown = mount(
        <TestDropdown
            emptyText="Empty"
            options={ [] } />
    );

    expect(dropdown.text()).toBe('Empty');
});

[
    { name: 'should render empty text if value is undefined', data: { value: undefined } },
    { name: 'should render empty text if value is null', data: { value: null } },
    { name: 'should render empty text if value is empty string', data: { value: '' } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const dropdown = mount(
            <TestDropdown
                emptyText="Empty"
                options={ getOptions() }
                value={ fixture.data.value } />
        );

        expect(dropdown.text()).toBe('Empty');
    });
});

[
    { name: 'should render empty text if option value is undefined', data: { value: undefined } },
    { name: 'should render empty text if option value is null', data: { value: null } },
    { name: 'should render empty text if option value is empty string', data: { value: '' } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const dropdown = mount(
            <TestDropdown
                emptyText="Empty"
                options={ [dropdownValueOptionState({ value: fixture.data.value })] } />
        );

        expect(dropdown.text()).toBe('Empty');
    });
});

test('should render option text of dropdown value', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions() }
            value="b" />
    );

    expect(dropdown.text()).toBe('B');
});

test('should render option selected text of dropdown value', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions() }
            value="c" />
    );

    expect(dropdown.text()).toBe('C (selected)');
});

test('should expand collapsed dropdown when clicked', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions() } />
    );

    dropdown.simulate('click');

    expect(dropdown.state('expanded')).toBe(true);
});

[
    { name: 'should expand collapsed dropdown when using enter', data: { which: 13 } },
    { name: 'should expand collapsed dropdown when using space', data: { which: 32 } },
    { name: 'should expand collapsed dropdown when using down arrow', data: { which: 40 } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const dropdown = mount(
            <TestDropdown
                options={ getOptions() } />
        );

        dropdown.simulate('keydown', { which: fixture.data.which });

        expect(dropdown.state('expanded')).toBe(true);
    });
});

test('should not expand collapsed dropdown when clicked and disabled', () => {
    const dropdown = mount(
        <TestDropdown
            disabled={ true }
            options={ getOptions() } />
    );

    dropdown.simulate('click');

    expect(dropdown.state('expanded')).toBe(false);
});

[
    { name: 'should not expand collapsed dropdown when using enter and disabled', data: { which: 13 } },
    { name: 'should not expand collapsed dropdown when using space and disabled', data: { which: 32 } },
    { name: 'should not expand collapsed dropdown when using down arrow and disabled', data: { which: 40 } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const dropdown = mount(
            <TestDropdown
                disabled={ true }
                options={ getOptions() } />
        );

        dropdown.simulate('keydown', { which: fixture.data.which });

        expect(dropdown.state('expanded')).toBe(false);
    });
});

test('should collapse expanded dropdown when clicked', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions() } />
    );

    dropdown.simulate('click');
    dropdown.simulate('click');

    expect(dropdown.state('expanded')).toBe(false);
});

test('should collapse expanded dropdown when clicking outside', () => {
    const dropdown = mount(
        <TestOnClickOutsideDropdown
            options={ getOptions() } />
    );

    dropdown.simulate('click');
    document.dispatchEvent(new Event('mousedown'));

    // cannot access the state of the underlying component due to the onClickOutside HOC.
    const elements = dropdown.find('TestDropdownValueOption');
    expect(elements.length).toBe(0);
});

test('should collapse expanded dropdown when using escape', () => {
    const dropdown = mount(
        <TestDropdown />
    );

    dropdown.simulate('click');
    dropdown.simulate('keydown', { which: 27 });

    expect(dropdown.state('expanded')).toBe(false);
});

[
    { name: 'should collapse expanded dropdown when using enter', data: { which: 13 } },
    { name: 'should collapse expanded dropdown when using space', data: { which: 32 } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const dropdown = mount(
            <TestDropdown
                options={ getOptions() } />
        );

        dropdown.simulate('click');
        dropdown.simulate('keydown', { which: fixture.data.which });

        expect(dropdown.state('expanded')).toBe(false);
    });
});

test('should change value when clicking new option', () => {
    const onChange = jest.fn();
    const dropdown = mount(
        <TestDropdown
            onChange={ onChange }
            options={ getOptions() }
            value="a" />
    );

    dropdown.simulate('click');

    const options = dropdown.find('TestDropdownValueOption');
    expect(options.length).toBe(3);

    options.at(1).simulate('click');
    expect(onChange).toHaveBeenCalledWith('b');
});

[
    { name: 'should change value when using enter on new option', data: { which: 13 } },
    { name: 'should change value when using space on new option', data: { which: 32 } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const onChange = jest.fn();
        const dropdown = mount(
            <TestDropdown
                onChange={ onChange }
                options={ getOptions() }
                value="a" />
        );

        expandAt(dropdown, 1);
        dropdown.simulate('keydown', { which: fixture.data.which });

        expect(onChange).toHaveBeenCalledWith('b');
    });
});

[
    { name: 'should cancel when using enter on same option', data: { which: 13 } },
    { name: 'should cancel when using space on same option', data: { which: 32 } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const onChange = jest.fn();
        const dropdown = mount(
            <TestDropdown
                highlightedIndex={ 0 }
                onChange={ onChange }
                options={ getOptions() }
                value="a" />
        );

        expandAt(dropdown, 0);

        dropdown.simulate('keydown', { which: fixture.data.which });

        expect(onChange).not.toHaveBeenCalled();
        expect(dropdown.state('expanded')).toBe(false);
    });
});

test('should be able to highlight option by typing its value', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions() }
            value="a" />
    );

    dropdown.simulate('click');
    dropdown.simulate('keypress', { which: 66 }); // b

    expect(dropdown.state('highlightedIndex')).toBe(1);
});

test('should change highlighted index when using down arrow when expanded', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions() } />
    );

    expandAt(dropdown, 0);
    dropdown.simulate('keydown', { which: 40 });

    expect(dropdown.state('highlightedIndex')).toBe(1);
});

test('should change highlighted index when using up arrow when expanded', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions() } />
    );

    expandAt(dropdown, 2);
    dropdown.simulate('keydown', { which: 38 });

    expect(dropdown.state('highlightedIndex')).toBe(1);
});

test('should skip disabled options when using down arrow when expanded', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions(false, true, false) } />
    );

    expandAt(dropdown, 0);
    dropdown.simulate('keydown', { which: 40 });

    expect(dropdown.state('highlightedIndex')).toBe(2);
});

test('should skip disabled options when using up arrow when expanded', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions(false, true, false) } />
    );

    expandAt(dropdown, 2);
    dropdown.simulate('keydown', { which: 38 });

    expect(dropdown.state('highlightedIndex')).toBe(0);
});

test('should wrap around highlighted index when using down arrow when expanded and on last option', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions() } />
    );

    expandAt(dropdown, 2);
    dropdown.simulate('keydown', { which: 40 });

    expect(dropdown.state('highlightedIndex')).toBe(0);
});

test('should wrap around highlighted index when using up arrow when expanded and on first option ', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions() } />
    );

    expandAt(dropdown, 0);
    dropdown.simulate('keydown', { which: 38 });

    expect(dropdown.state('highlightedIndex')).toBe(2);
});

[
    { name: 'should ignore up arrow key when all options are disabled', data: { which: 38 } },
    { name: 'should ignore down arrow key when all options are disabled', data: { which: 40 } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const dropdown = mount(
            <TestDropdown
                options={ getOptions(true, true, true) } />
        );

        expandAt(dropdown, 0);
        dropdown.simulate('keydown', { which: fixture.data.which });

        expect(dropdown.state('highlightedIndex')).toBe(0);
    });
});

test('should select first if no initial value when expanded and using down arrow ', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions() } />
    );

    expandAt(dropdown);
    dropdown.simulate('keydown', { which: 40 });

    expect(dropdown.state('highlightedIndex')).toBe(0);
});

test('should select last if no initial value when expanded and using up arrow ', () => {
    const dropdown = mount(
        <TestDropdown
            options={ getOptions() } />
    );

    expandAt(dropdown);
    dropdown.simulate('keydown', { which: 38 });

    expect(dropdown.state('highlightedIndex')).toBe(2);
});

[
    { name: 'should ignore up arrow key when all options are disabled', data: { which: 38 } },
    { name: 'should ignore down arrow key when all options are disabled', data: { which: 40 } }
].forEach((fixture) => {
    test(fixture.name, () => {
        const dropdown = mount(
            <TestDropdown
                options={ [] } />
        );

        expandAt(dropdown);
        dropdown.simulate('keydown', { which: fixture.data.which });

        expect(dropdown.state('highlightedIndex')).toBe(-1);
    });
});
